import wave
import pathlib
import random
import subprocess

try:
    import sounddevice
    import soundfile
except ImportError:
    sounddevice = None

from primitive_tts.en import MAP


DEBUG = False


def string_to_tokens(text):
    """
    Convert a string to a list of tokens.
    """

    # TODO: Remove non-alphabetic characters

    return text.lower().split(' ')


def word_to_chunks(word):
    """
    Break a word into chunks of 2 characters.
    """
    chunks = []

    # Even length words can be split into 2 character chunks
    if len(word) % 2 != 0:
        # Prefer 2 character chunks
        # Pick a random character to double up
        double_index = random.choice(range(len(word)))
        character = word[double_index]
        word = word[:double_index] + character + word[double_index:]

    chunks = [word[i:i + 2] for i in range(0, len(word), 2)]

    return chunks


def word_to_phonemes(word_chunks):
    """
    Return the phonemes for a word in a given language.
    """

    chunks = []

    for chunk in word_chunks:
        chunks.extend(MAP[chunk])

    return chunks


def speak(text, language='en', output_file='output.wav'):
    """
    Speak the provided text.
    """
    words = string_to_tokens(text)

    phonemes = []

    for index, word in enumerate(words):
        word_chunks = word_to_chunks(word)
        if DEBUG:
            print('word_chunks >>>', word_chunks)
        word_phonemes = word_to_phonemes(word_chunks)
        for word_phoneme in word_phonemes:
            # Avoid consecutive duplicate phonemes for smoother sound
            # This includes phonemes that end with the letter that the next phoneme starts with
            if len(phonemes) == 0 or (phonemes[-1] != word_phoneme and phonemes[-1][-1] != word_phoneme[0]):
                phonemes.append(word_phoneme)

        if index < len(words) - 1:
            phonemes.append('__')

    if DEBUG:
        print('phonemes >>>', ''.join([f'{p}-' for p in phonemes]))

    parent_dir = pathlib.Path(__file__).parent.resolve()
    sample_dir = parent_dir / 'phonemes' / language

    sample_rate = 44100  # Samples per second
    num_channels = 1  # Mono audio
    sample_width = 2  # 16-bit audio (2 bytes per sample)
    num_frames = 0  # Initially, the file is empty

    with wave.open(output_file, 'wb') as wav_out:
        wav_out.setparams(
            (num_channels, sample_width, sample_rate, num_frames, 'NONE', 'not compressed')
        )

        for phoneme in phonemes:
            word_file = f'{phoneme}.wav'
            wav_path = str(sample_dir / word_file)

            try:
                with wave.open(wav_path, 'rb') as wav_in:
                    if not wav_out.getnframes():
                        wav_out.setparams(wav_in.getparams())
                    wav_out.writeframes(wav_in.readframes(wav_in.getnframes()))
            except FileNotFoundError as e:
                raise Exception(f'Entry for {word_file} does not exist') from e

        if sounddevice is None:
            # Fall back to subprocess if sounddevice is not available
            subprocess.run(['aplay', output_file])
        else:
            data, fs = soundfile.read(output_file)
            sounddevice.play(data, fs)
            sounddevice.wait()



from enum import auto, StrEnum


class Phonemes(StrEnum):
    AA = auto()
    AE = auto()
    AH = auto()
    AO = auto()
    AW = auto()
    AY = auto()
    B = auto()
    CH = auto()
    D = auto()
    DH = auto()
    EH = auto()
    ER = auto()
    EY = auto()
    F = auto()
    G = auto()
    HH = auto()
    IH = auto()
    IY = auto()
    JH = auto()
    K = auto()
    L = auto()
    M = auto()
    N = auto()
    NG = auto()
    OW = auto()
    OY = auto()
    P = auto()
    R = auto()
    S = auto()
    SH = auto()
    T = auto()
    TH = auto()
    UH = auto()
    UW = auto()
    V = auto()
    W = auto()
    Y = auto()
    Z = auto()
    ZH = auto()


MAP = {
    "aa": [Phonemes.AA],
    "ab": [Phonemes.AA, Phonemes.B],
    "ac": [Phonemes.AA, Phonemes.K],
    "ad": [Phonemes.AA, Phonemes.D],
    "ae": [Phonemes.AE],
    "af": [Phonemes.AA, Phonemes.F],
    "ag": [Phonemes.AA, Phonemes.G],
    "ah": [Phonemes.AA, Phonemes.HH],
    "ai": [Phonemes.EY],
    "aj": [Phonemes.AA, Phonemes.JH],
    "ak": [Phonemes.AA, Phonemes.K],
    "al": [Phonemes.AA, Phonemes.L],
    "am": [Phonemes.AA, Phonemes.M],
    "an": [Phonemes.AH, Phonemes.N],
    "ao": [Phonemes.AO],
    "ap": [Phonemes.AH, Phonemes.P],
    "aq": [Phonemes.AH, Phonemes.K],
    "ar": [Phonemes.AA, Phonemes.R],
    "as": [Phonemes.AH, Phonemes.S],
    "at": [Phonemes.AH, Phonemes.T],
    "au": [Phonemes.AW],
    "av": [Phonemes.AA, Phonemes.V],
    "aw": [Phonemes.AW],
    "ax": [Phonemes.AA, Phonemes.K, Phonemes.S],
    "ay": [Phonemes.AY],
    "az": [Phonemes.AA, Phonemes.Z],
    "ba": [Phonemes.B, Phonemes.AA],
    "bb": [Phonemes.B],
    "bc": [Phonemes.B, Phonemes.K],
    "bd": [Phonemes.B, Phonemes.D],
    "be": [Phonemes.B, Phonemes.IY],
    "bf": [Phonemes.B, Phonemes.F],
    "bg": [Phonemes.B, Phonemes.G],
    "bh": [Phonemes.B, Phonemes.HH],
    "bi": [Phonemes.B, Phonemes.IY],
    "bj": [Phonemes.B, Phonemes.JH],
    "bk": [Phonemes.B, Phonemes.K],
    "bl": [Phonemes.B, Phonemes.L],
    "bm": [Phonemes.B, Phonemes.M],
    "bn": [Phonemes.B, Phonemes.N],
    "bo": [Phonemes.B, Phonemes.OW],
    "bp": [Phonemes.B, Phonemes.P],
    "bq": [Phonemes.B, Phonemes.K],
    "br": [Phonemes.B, Phonemes.R],
    "bs": [Phonemes.B, Phonemes.S],
    "bt": [Phonemes.B, Phonemes.T],
    "bu": [Phonemes.B, Phonemes.UH],
    "bv": [Phonemes.B, Phonemes.V],
    "bw": [Phonemes.B, Phonemes.UW],
    "bx": [Phonemes.B, Phonemes.K, Phonemes.S],
    "by": [Phonemes.B, Phonemes.AY],
    "bz": [Phonemes.B, Phonemes.Z],
    "ca": [Phonemes.K, Phonemes.AH],
    "cb": [Phonemes.K, Phonemes.B],
    "cc": [Phonemes.K],
    "cd": [Phonemes.K, Phonemes.D],
    "ce": [Phonemes.K, Phonemes.IY],
    "cf": [Phonemes.K, Phonemes.F],
    "cg": [Phonemes.K, Phonemes.G],
    "ch": [Phonemes.CH],
    "ci": [Phonemes.SH, Phonemes.IY],
    "cj": [Phonemes.K, Phonemes.JH],
    "ck": [Phonemes.K],
    "cl": [Phonemes.K, Phonemes.L],
    "cm": [Phonemes.K, Phonemes.M],
    "cn": [Phonemes.K, Phonemes.N],
    "co": [Phonemes.K, Phonemes.OW],
    "cp": [Phonemes.K, Phonemes.P],
    "cq": [Phonemes.K],
    "cr": [Phonemes.K, Phonemes.R],
    "cs": [Phonemes.K, Phonemes.S],
    "ct": [Phonemes.K, Phonemes.T],
    "cu": [Phonemes.K, Phonemes.UW],
    "cv": [Phonemes.K, Phonemes.V],
    "cw": [Phonemes.K, Phonemes.UW],
    "cx": [Phonemes.K, Phonemes.S],
    "cy": [Phonemes.K, Phonemes.AY],
    "cz": [Phonemes.K, Phonemes.Z],
    "da": [Phonemes.D, Phonemes.AA],
    "db": [Phonemes.D, Phonemes.B],
    "dc": [Phonemes.D, Phonemes.K],
    "dd": [Phonemes.D],
    "de": [Phonemes.D, Phonemes.EH],
    "df": [Phonemes.D, Phonemes.F],
    "dg": [Phonemes.D, Phonemes.G],
    "dh": [Phonemes.D, Phonemes.HH],
    "di": [Phonemes.D, Phonemes.IY],
    "dj": [Phonemes.D, Phonemes.JH],
    "dk": [Phonemes.D, Phonemes.K],
    "dl": [Phonemes.D, Phonemes.L],
    "dm": [Phonemes.D, Phonemes.M],
    "dn": [Phonemes.D, Phonemes.N],
    "do": [Phonemes.D, Phonemes.OW],
    "dp": [Phonemes.D, Phonemes.P],
    "dq": [Phonemes.D, Phonemes.K],
    "dr": [Phonemes.D, Phonemes.R],
    "ds": [Phonemes.D, Phonemes.Z],
    "dt": [Phonemes.D, Phonemes.T],
    "du": [Phonemes.D, Phonemes.UW],
    "dv": [Phonemes.D, Phonemes.V],
    "dw": [Phonemes.D, Phonemes.UW],
    "dx": [Phonemes.D, Phonemes.K, Phonemes.S],
    "dy": [Phonemes.D, Phonemes.AY],
    "dz": [Phonemes.D, Phonemes.Z],
    "ea": [Phonemes.AE],
    "eb": [Phonemes.AH, Phonemes.B],
    "ec": [Phonemes.AH, Phonemes.K],
    "ed": [Phonemes.AE, Phonemes.D],
    "ee": [Phonemes.EY],
    "ef": [Phonemes.AH, Phonemes.F],
    "eg": [Phonemes.AH, Phonemes.G],
    "eh": [Phonemes.AH, Phonemes.HH],
    "ei": [Phonemes.AH],
    "ej": [Phonemes.AH, Phonemes.JH],
    "ek": [Phonemes.AH, Phonemes.K],
    "el": [Phonemes.EH, Phonemes.L],
    "em": [Phonemes.AH, Phonemes.M],
    "en": [Phonemes.AH, Phonemes.N],
    "eo": [Phonemes.AH, Phonemes.OW],
    "ep": [Phonemes.AH, Phonemes.P],
    "eq": [Phonemes.AH, Phonemes.K],
    "er": [Phonemes.ER],
    "es": [Phonemes.AH, Phonemes.S],
    "et": [Phonemes.AH, Phonemes.T],
    "eu": [Phonemes.AH, Phonemes.UW],
    "ev": [Phonemes.AH, Phonemes.V],
    "ew": [Phonemes.AH, Phonemes.UW],
    "ex": [Phonemes.AH, Phonemes.K, Phonemes.S],
    "ey": [Phonemes.IY],
    "ez": [Phonemes.AH, Phonemes.Z],
    "fa": [Phonemes.F, Phonemes.AA],
    "fb": [Phonemes.F, Phonemes.B],
    "fc": [Phonemes.F, Phonemes.K],
    "fd": [Phonemes.F, Phonemes.D],
    "fe": [Phonemes.F, Phonemes.IY],
    "ff": [Phonemes.F],
    "fg": [Phonemes.F, Phonemes.G],
    "fh": [Phonemes.F, Phonemes.HH],
    "fi": [Phonemes.F, Phonemes.IY],
    "fj": [Phonemes.F, Phonemes.JH],
    "fk": [Phonemes.F, Phonemes.K],
    "fl": [Phonemes.F, Phonemes.L],
    "fm": [Phonemes.F, Phonemes.M],
    "fn": [Phonemes.F, Phonemes.N],
    "fo": [Phonemes.F, Phonemes.OW],
    "fp": [Phonemes.F, Phonemes.P],
    "fq": [Phonemes.F, Phonemes.K],
    "fr": [Phonemes.F, Phonemes.R],
    "fs": [Phonemes.F, Phonemes.S],
    "ft": [Phonemes.F, Phonemes.T],
    "fu": [Phonemes.F, Phonemes.UW],
    "fv": [Phonemes.F, Phonemes.V],
    "fw": [Phonemes.F, Phonemes.UW],
    "fx": [Phonemes.F, Phonemes.K, Phonemes.S],
    "fy": [Phonemes.F, Phonemes.AY],
    "fz": [Phonemes.F, Phonemes.Z],
    "ga": [Phonemes.G, Phonemes.AA],
    "gb": [Phonemes.G, Phonemes.B],
    "gc": [Phonemes.G, Phonemes.K],
    "gd": [Phonemes.G, Phonemes.D],
    "ge": [Phonemes.G, Phonemes.AH],
    "gf": [Phonemes.G, Phonemes.F],
    "gg": [Phonemes.G],
    "gh": [Phonemes.G, Phonemes.HH],
    "gi": [Phonemes.G, Phonemes.IY],
    "gj": [Phonemes.G, Phonemes.JH],
    "gk": [Phonemes.G, Phonemes.K],
    "gl": [Phonemes.G, Phonemes.L],
    "gm": [Phonemes.G, Phonemes.M],
    "gn": [Phonemes.G, Phonemes.N],
    "go": [Phonemes.G, Phonemes.OW],
    "gp": [Phonemes.G, Phonemes.P],
    "gq": [Phonemes.G, Phonemes.K],
    "gr": [Phonemes.G, Phonemes.R],
    "gs": [Phonemes.G, Phonemes.S],
    "gt": [Phonemes.G, Phonemes.T],
    "gu": [Phonemes.G, Phonemes.UW],
    "gv": [Phonemes.G, Phonemes.V],
    "gw": [Phonemes.G, Phonemes.UW],
    "gx": [Phonemes.G, Phonemes.K, Phonemes.S],
    "gy": [Phonemes.G, Phonemes.AY],
    "gz": [Phonemes.G, Phonemes.Z],
    "ha": [Phonemes.HH, Phonemes.AA],
    "hb": [Phonemes.HH, Phonemes.B],
    "hc": [Phonemes.HH, Phonemes.K],
    "hd": [Phonemes.HH, Phonemes.D],
    "he": [Phonemes.HH, Phonemes.IY],
    "hf": [Phonemes.HH, Phonemes.F],
    "hg": [Phonemes.HH, Phonemes.G],
    "hh": [Phonemes.HH],
    "hi": [Phonemes.HH, Phonemes.IY],
    "hj": [Phonemes.HH, Phonemes.JH],
    "hk": [Phonemes.HH, Phonemes.K],
    "hl": [Phonemes.HH, Phonemes.L],
    "hm": [Phonemes.HH, Phonemes.M],
    "hn": [Phonemes.HH, Phonemes.N],
    "ho": [Phonemes.HH, Phonemes.OW],
    "hp": [Phonemes.HH, Phonemes.P],
    "hq": [Phonemes.HH, Phonemes.K],
    "hr": [Phonemes.HH, Phonemes.R],
    "hs": [Phonemes.HH, Phonemes.S],
    "ht": [Phonemes.HH, Phonemes.T],
    "hu": [Phonemes.HH, Phonemes.UW],
    "hv": [Phonemes.HH, Phonemes.V],
    "hw": [Phonemes.HH, Phonemes.UW],
    "hx": [Phonemes.HH, Phonemes.K, Phonemes.S],
    "hy": [Phonemes.HH, Phonemes.AY],
    "hz": [Phonemes.HH, Phonemes.Z],
    "ia": [Phonemes.IY],
    "ib": [Phonemes.EH, Phonemes.B],
    "ic": [Phonemes.IY, Phonemes.K],
    "id": [Phonemes.IY, Phonemes.D],
    "ie": [Phonemes.IY],
    "if": [Phonemes.IY, Phonemes.F],
    "ig": [Phonemes.IY, Phonemes.G],
    "ih": [Phonemes.IY, Phonemes.HH],
    "ii": [Phonemes.AH, Phonemes.IY],
    "ij": [Phonemes.IY, Phonemes.JH],
    "ik": [Phonemes.IY, Phonemes.K],
    "il": [Phonemes.IY, Phonemes.L],
    "im": [Phonemes.IY, Phonemes.M],
    "in": [Phonemes.IH, Phonemes.N],
    "io": [Phonemes.IY, Phonemes.OW],
    "ip": [Phonemes.IY, Phonemes.P],
    "iq": [Phonemes.IY, Phonemes.K],
    "ir": [Phonemes.IY, Phonemes.R],
    "is": [Phonemes.IY, Phonemes.S],
    "it": [Phonemes.IY, Phonemes.T],
    "iu": [Phonemes.IY, Phonemes.UW],
    "iv": [Phonemes.IY, Phonemes.V],
    "iw": [Phonemes.IY, Phonemes.UW],
    "ix": [Phonemes.IY, Phonemes.K, Phonemes.S],
    "iy": [Phonemes.IY],
    "iz": [Phonemes.IY, Phonemes.Z],
    "ja": [Phonemes.JH, Phonemes.AA],
    "jb": [Phonemes.JH, Phonemes.B],
    "jc": [Phonemes.JH, Phonemes.K],
    "jd": [Phonemes.JH, Phonemes.D],
    "je": [Phonemes.JH, Phonemes.IY],
    "jf": [Phonemes.JH, Phonemes.F],
    "jg": [Phonemes.JH, Phonemes.G],
    "jh": [Phonemes.JH],
    "ji": [Phonemes.JH, Phonemes.IY],
    "jj": [Phonemes.JH],
    "jk": [Phonemes.JH, Phonemes.K],
    "jl": [Phonemes.JH, Phonemes.L],
    "jm": [Phonemes.JH, Phonemes.M],
    "jn": [Phonemes.JH, Phonemes.N],
    "jo": [Phonemes.JH, Phonemes.OW],
    "jp": [Phonemes.JH, Phonemes.P],
    "jq": [Phonemes.JH, Phonemes.K],
    "jr": [Phonemes.JH, Phonemes.R],
    "js": [Phonemes.JH, Phonemes.S],
    "jt": [Phonemes.JH, Phonemes.T],
    "ju": [Phonemes.JH, Phonemes.AH],
    "jv": [Phonemes.JH, Phonemes.V],
    "jw": [Phonemes.JH, Phonemes.UW],
    "jx": [Phonemes.JH, Phonemes.K, Phonemes.S],
    "jy": [Phonemes.JH, Phonemes.AY],
    "jz": [Phonemes.JH, Phonemes.Z],
    "ka": [Phonemes.K, Phonemes.AA],
    "kb": [Phonemes.K, Phonemes.B],
    "kc": [Phonemes.K, Phonemes.K],
    "kd": [Phonemes.K, Phonemes.D],
    "ke": [Phonemes.K, Phonemes.IY],
    "kf": [Phonemes.K, Phonemes.F],
    "kg": [Phonemes.K, Phonemes.G],
    "kh": [Phonemes.K, Phonemes.HH],
    "ki": [Phonemes.K, Phonemes.IY],
    "kj": [Phonemes.K, Phonemes.JH],
    "kk": [Phonemes.K],
    "kl": [Phonemes.K, Phonemes.L],
    "km": [Phonemes.K, Phonemes.M],
    "kn": [Phonemes.K, Phonemes.N],
    "ko": [Phonemes.K, Phonemes.OW],
    "kp": [Phonemes.K, Phonemes.P],
    "kq": [Phonemes.K, Phonemes.K],
    "kr": [Phonemes.K, Phonemes.R],
    "ks": [Phonemes.K, Phonemes.S],
    "kt": [Phonemes.K, Phonemes.T],
    "ku": [Phonemes.K, Phonemes.UW],
    "kv": [Phonemes.K, Phonemes.V],
    "kw": [Phonemes.K, Phonemes.UW],
    "kx": [Phonemes.K, Phonemes.K, Phonemes.S],
    "ky": [Phonemes.K, Phonemes.AY],
    "kz": [Phonemes.K, Phonemes.Z],
    "la": [Phonemes.L, Phonemes.AA],
    "lb": [Phonemes.L, Phonemes.B],
    "lc": [Phonemes.L, Phonemes.K],
    "ld": [Phonemes.L, Phonemes.D],
    "le": [Phonemes.L, Phonemes.IY],
    "lf": [Phonemes.L, Phonemes.F],
    "lg": [Phonemes.L, Phonemes.G],
    "lh": [Phonemes.L, Phonemes.HH],
    "li": [Phonemes.L, Phonemes.EY],
    "lj": [Phonemes.L, Phonemes.JH],
    "lk": [Phonemes.L, Phonemes.K],
    "ll": [Phonemes.L],
    "lm": [Phonemes.L, Phonemes.M],
    "ln": [Phonemes.L, Phonemes.N],
    "lo": [Phonemes.L, Phonemes.OW],
    "lp": [Phonemes.L, Phonemes.P],
    "lq": [Phonemes.L, Phonemes.K],
    "lr": [Phonemes.L, Phonemes.R],
    "ls": [Phonemes.L, Phonemes.S],
    "lt": [Phonemes.L, Phonemes.T],
    "lu": [Phonemes.L, Phonemes.UW],
    "lv": [Phonemes.L, Phonemes.V],
    "lw": [Phonemes.L, Phonemes.UW],
    "lx": [Phonemes.L, Phonemes.K, Phonemes.S],
    "ly": [Phonemes.L, Phonemes.AY],
    "lz": [Phonemes.L, Phonemes.Z],
    "ma": [Phonemes.M, Phonemes.AA],
    "mb": [Phonemes.M, Phonemes.B],
    "mc": [Phonemes.M, Phonemes.K],
    "md": [Phonemes.M, Phonemes.D],
    "me": [Phonemes.M, Phonemes.IY],
    "mf": [Phonemes.M, Phonemes.F],
    "mg": [Phonemes.M, Phonemes.G],
    "mh": [Phonemes.M, Phonemes.HH],
    "mi": [Phonemes.M, Phonemes.IY],
    "mj": [Phonemes.M, Phonemes.JH],
    "mk": [Phonemes.M, Phonemes.K],
    "ml": [Phonemes.M, Phonemes.L],
    "mm": [Phonemes.M],
    "mn": [Phonemes.M, Phonemes.N],
    "mo": [Phonemes.M, Phonemes.OW],
    "mp": [Phonemes.M, Phonemes.P],
    "mq": [Phonemes.M, Phonemes.K],
    "mr": [Phonemes.M, Phonemes.R],
    "ms": [Phonemes.M, Phonemes.S],
    "mt": [Phonemes.M, Phonemes.T],
    "mu": [Phonemes.M, Phonemes.UW],
    "mv": [Phonemes.M, Phonemes.V],
    "mw": [Phonemes.M, Phonemes.UW],
    "mx": [Phonemes.M, Phonemes.K, Phonemes.S],
    "my": [Phonemes.M, Phonemes.AH, Phonemes.IY],
    "mz": [Phonemes.M, Phonemes.Z],
    "na": [Phonemes.N, Phonemes.AA],
    "nb": [Phonemes.N, Phonemes.B],
    "nc": [Phonemes.N, Phonemes.K],
    "nd": [Phonemes.N, Phonemes.D],
    "ne": [Phonemes.N, Phonemes.EH],
    "nf": [Phonemes.N, Phonemes.F],
    "ng": [Phonemes.N, Phonemes.G],
    "nh": [Phonemes.N, Phonemes.HH],
    "ni": [Phonemes.N, Phonemes.IY],
    "nj": [Phonemes.N, Phonemes.JH],
    "nk": [Phonemes.N, Phonemes.K],
    "nl": [Phonemes.N, Phonemes.L],
    "nm": [Phonemes.N, Phonemes.M],
    "nn": [Phonemes.N],
    "no": [Phonemes.N, Phonemes.OW],
    "np": [Phonemes.N, Phonemes.P],
    "nq": [Phonemes.N, Phonemes.K],
    "nr": [Phonemes.N, Phonemes.R],
    "ns": [Phonemes.N, Phonemes.S],
    "nt": [Phonemes.N, Phonemes.T],
    "nu": [Phonemes.N, Phonemes.UW],
    "nv": [Phonemes.N, Phonemes.V],
    "nw": [Phonemes.N, Phonemes.UW],
    "nx": [Phonemes.N, Phonemes.K, Phonemes.S],
    "ny": [Phonemes.N, Phonemes.AY],
    "nz": [Phonemes.N, Phonemes.Z],
    "oa": [Phonemes.OW, Phonemes.AA],
    "ob": [Phonemes.OW, Phonemes.B],
    "oc": [Phonemes.OW, Phonemes.K],
    "od": [Phonemes.OW, Phonemes.D],
    "oe": [Phonemes.OW, Phonemes.IY],
    "of": [Phonemes.OW, Phonemes.F],
    "og": [Phonemes.OW, Phonemes.G],
    "oh": [Phonemes.OW, Phonemes.HH],
    "oi": [Phonemes.OW, Phonemes.IY],
    "oj": [Phonemes.OW, Phonemes.JH],
    "ok": [Phonemes.OW, Phonemes.K],
    "ol": [Phonemes.OW, Phonemes.L],
    "om": [Phonemes.OW, Phonemes.M],
    "on": [Phonemes.AH, Phonemes.N],
    "oo": [Phonemes.OW],
    "op": [Phonemes.OW, Phonemes.P],
    "oq": [Phonemes.OW, Phonemes.K],
    "or": [Phonemes.OW, Phonemes.R],
    "os": [Phonemes.OW, Phonemes.S],
    "ot": [Phonemes.OW, Phonemes.T],
    "ou": [Phonemes.OW, Phonemes.UW],
    "ov": [Phonemes.OW, Phonemes.V],
    "ow": [Phonemes.OW, Phonemes.UW],
    "ox": [Phonemes.OW, Phonemes.K, Phonemes.S],
    "oy": [Phonemes.OW, Phonemes.AY],
    "oz": [Phonemes.OW, Phonemes.Z],
    "pa": [Phonemes.P, Phonemes.AA],
    "pb": [Phonemes.P, Phonemes.B],
    "pc": [Phonemes.P, Phonemes.K],
    "pd": [Phonemes.P, Phonemes.D],
    "pe": [Phonemes.P, Phonemes.EH],
    "pf": [Phonemes.P, Phonemes.F],
    "pg": [Phonemes.P, Phonemes.G],
    "ph": [Phonemes.P, Phonemes.HH],
    "pi": [Phonemes.P, Phonemes.IY],
    "pj": [Phonemes.P, Phonemes.JH],
    "pk": [Phonemes.P, Phonemes.K],
    "pl": [Phonemes.P, Phonemes.L],
    "pm": [Phonemes.P, Phonemes.M],
    "pn": [Phonemes.P, Phonemes.N],
    "po": [Phonemes.P, Phonemes.OW],
    "pp": [Phonemes.P],
    "pq": [Phonemes.P, Phonemes.K],
    "pr": [Phonemes.P, Phonemes.R],
    "ps": [Phonemes.P, Phonemes.S],
    "pt": [Phonemes.P, Phonemes.T],
    "pu": [Phonemes.P, Phonemes.UW],
    "pv": [Phonemes.P, Phonemes.V],
    "pw": [Phonemes.P, Phonemes.UW],
    "px": [Phonemes.P, Phonemes.K, Phonemes.S],
    "py": [Phonemes.P, Phonemes.AY],
    "pz": [Phonemes.P, Phonemes.Z],
    "qa": [Phonemes.K, Phonemes.AA],
    "qb": [Phonemes.K, Phonemes.B],
    "qc": [Phonemes.K, Phonemes.K],
    "qd": [Phonemes.K, Phonemes.D],
    "qe": [Phonemes.K, Phonemes.IY],
    "qf": [Phonemes.K, Phonemes.F],
    "qg": [Phonemes.K, Phonemes.G],
    "qh": [Phonemes.K, Phonemes.HH],
    "qi": [Phonemes.K, Phonemes.IY],
    "qj": [Phonemes.K, Phonemes.JH],
    "qk": [Phonemes.K],
    "ql": [Phonemes.K, Phonemes.L],
    "qm": [Phonemes.K, Phonemes.M],
    "qn": [Phonemes.K, Phonemes.N],
    "qo": [Phonemes.K, Phonemes.OW],
    "qp": [Phonemes.K, Phonemes.P],
    "qq": [Phonemes.K],
    "qr": [Phonemes.K, Phonemes.R],
    "qs": [Phonemes.K, Phonemes.S],
    "qt": [Phonemes.K, Phonemes.T],
    "qu": [Phonemes.K, Phonemes.W],
    "qv": [Phonemes.K, Phonemes.V],
    "qw": [Phonemes.K, Phonemes.UW],
    "qx": [Phonemes.K, Phonemes.K, Phonemes.S],
    "qy": [Phonemes.K, Phonemes.AY],
    "qz": [Phonemes.K, Phonemes.Z],
    "ra": [Phonemes.R, Phonemes.AA],
    "rb": [Phonemes.R, Phonemes.B],
    "rc": [Phonemes.R, Phonemes.K],
    "rd": [Phonemes.R, Phonemes.D],
    "re": [Phonemes.R, Phonemes.IY],
    "rf": [Phonemes.R, Phonemes.F],
    "rg": [Phonemes.R, Phonemes.G],
    "rh": [Phonemes.R, Phonemes.HH],
    "ri": [Phonemes.R, Phonemes.IY],
    "rj": [Phonemes.R, Phonemes.JH],
    "rk": [Phonemes.R, Phonemes.K],
    "rl": [Phonemes.R, Phonemes.L],
    "rm": [Phonemes.R, Phonemes.M],
    "rn": [Phonemes.R, Phonemes.N],
    "ro": [Phonemes.R, Phonemes.OW],
    "rp": [Phonemes.R, Phonemes.P],
    "rq": [Phonemes.R, Phonemes.K],
    "rr": [Phonemes.R],
    "rs": [Phonemes.R, Phonemes.S],
    "rt": [Phonemes.R, Phonemes.T],
    "ru": [Phonemes.R, Phonemes.UW],
    "rv": [Phonemes.R, Phonemes.V],
    "rw": [Phonemes.R, Phonemes.UW],
    "rx": [Phonemes.R, Phonemes.K, Phonemes.S],
    "ry": [Phonemes.R, Phonemes.AY],
    "rz": [Phonemes.R, Phonemes.Z],
    "sa": [Phonemes.S, Phonemes.AA],
    "sb": [Phonemes.S, Phonemes.B],
    "sc": [Phonemes.S, Phonemes.K],
    "sd": [Phonemes.S, Phonemes.D],
    "se": [Phonemes.S, Phonemes.EY],
    "sf": [Phonemes.S, Phonemes.F],
    "sg": [Phonemes.S, Phonemes.G],
    "sh": [Phonemes.SH],
    "si": [Phonemes.S, Phonemes.IY],
    "sj": [Phonemes.S, Phonemes.JH],
    "sk": [Phonemes.S, Phonemes.K],
    "sl": [Phonemes.S, Phonemes.L],
    "sm": [Phonemes.S, Phonemes.M],
    "sn": [Phonemes.S, Phonemes.N],
    "so": [Phonemes.S, Phonemes.OW],
    "sp": [Phonemes.S, Phonemes.P],
    "sq": [Phonemes.S, Phonemes.K],
    "sr": [Phonemes.S, Phonemes.R],
    "ss": [Phonemes.S],
    "st": [Phonemes.S, Phonemes.T],
    "su": [Phonemes.S, Phonemes.UW],
    "sv": [Phonemes.S, Phonemes.V],
    "sw": [Phonemes.S, Phonemes.UW],
    "sx": [Phonemes.S, Phonemes.K, Phonemes.S],
    "sy": [Phonemes.S, Phonemes.AY],
    "sz": [Phonemes.S, Phonemes.Z],
    "ta": [Phonemes.T, Phonemes.AA],
    "tb": [Phonemes.T, Phonemes.B],
    "tc": [Phonemes.T, Phonemes.K],
    "td": [Phonemes.T, Phonemes.D],
    "te": [Phonemes.T, Phonemes.EH],
    "tf": [Phonemes.T, Phonemes.F],
    "tg": [Phonemes.T, Phonemes.G],
    "th": [Phonemes.TH],
    "ti": [Phonemes.T, Phonemes.EY],
    "tj": [Phonemes.T, Phonemes.JH],
    "tk": [Phonemes.T, Phonemes.K],
    "tl": [Phonemes.T, Phonemes.L],
    "tm": [Phonemes.T, Phonemes.M],
    "tn": [Phonemes.T, Phonemes.N],
    "to": [Phonemes.T, Phonemes.OW],
    "tp": [Phonemes.T, Phonemes.P],
    "tq": [Phonemes.T, Phonemes.K],
    "tr": [Phonemes.T, Phonemes.R],
    "ts": [Phonemes.T, Phonemes.S],
    "tt": [Phonemes.T],
    "tu": [Phonemes.T, Phonemes.UW],
    "tv": [Phonemes.T, Phonemes.V],
    "tw": [Phonemes.T, Phonemes.UW],
    "tx": [Phonemes.T, Phonemes.K, Phonemes.S],
    "ty": [Phonemes.T, Phonemes.AY],
    "tz": [Phonemes.T, Phonemes.Z],
    "ua": [Phonemes.UW, Phonemes.AA],
    "ub": [Phonemes.UW, Phonemes.B],
    "uc": [Phonemes.UW, Phonemes.K],
    "ud": [Phonemes.UW, Phonemes.D],
    "ue": [Phonemes.UW, Phonemes.IY],
    "uf": [Phonemes.UW, Phonemes.F],
    "ug": [Phonemes.AH, Phonemes.G],
    "uh": [Phonemes.UW, Phonemes.HH],
    "ui": [Phonemes.UW, Phonemes.IY],
    "uj": [Phonemes.UW, Phonemes.JH],
    "uk": [Phonemes.UW, Phonemes.K],
    "ul": [Phonemes.UW, Phonemes.L],
    "um": [Phonemes.UW, Phonemes.M],
    "un": [Phonemes.UW, Phonemes.N],
    "uo": [Phonemes.UW, Phonemes.OW],
    "up": [Phonemes.AH, Phonemes.P],
    "uq": [Phonemes.UW, Phonemes.K],
    "ur": [Phonemes.UW, Phonemes.R],
    "us": [Phonemes.UW, Phonemes.S],
    "ut": [Phonemes.UW, Phonemes.T],
    "uu": [Phonemes.UW],
    "uv": [Phonemes.UW, Phonemes.V],
    "uw": [Phonemes.UW],
    "ux": [Phonemes.UW, Phonemes.K, Phonemes.S],
    "uy": [Phonemes.UW, Phonemes.AY],
    "uz": [Phonemes.UW, Phonemes.Z],
    "va": [Phonemes.V, Phonemes.AA],
    "vb": [Phonemes.V, Phonemes.B],
    "vc": [Phonemes.V, Phonemes.K],
    "vd": [Phonemes.V, Phonemes.D],
    "ve": [Phonemes.V, Phonemes.IY],
    "vf": [Phonemes.V, Phonemes.F],
    "vg": [Phonemes.V, Phonemes.G],
    "vh": [Phonemes.V, Phonemes.HH],
    "vi": [Phonemes.V, Phonemes.IY],
    "vj": [Phonemes.V, Phonemes.JH],
    "vk": [Phonemes.V, Phonemes.K],
    "vl": [Phonemes.V, Phonemes.L],
    "vm": [Phonemes.V, Phonemes.M],
    "vn": [Phonemes.V, Phonemes.N],
    "vo": [Phonemes.V, Phonemes.OW],
    "vp": [Phonemes.V, Phonemes.P],
    "vq": [Phonemes.V, Phonemes.K],
    "vr": [Phonemes.V, Phonemes.R],
    "vs": [Phonemes.V, Phonemes.S],
    "vt": [Phonemes.V, Phonemes.T],
    "vu": [Phonemes.V, Phonemes.UW],
    "vv": [Phonemes.V],
    "vw": [Phonemes.V, Phonemes.UW],
    "vx": [Phonemes.V, Phonemes.K, Phonemes.S],
    "vy": [Phonemes.V, Phonemes.AY],
    "vz": [Phonemes.V, Phonemes.Z],
    "wa": [Phonemes.W, Phonemes.AH],
    "wb": [Phonemes.W, Phonemes.B],
    "wc": [Phonemes.W, Phonemes.K],
    "wd": [Phonemes.W, Phonemes.D],
    "we": [Phonemes.W, Phonemes.IY],
    "wf": [Phonemes.W, Phonemes.F],
    "wg": [Phonemes.W, Phonemes.G],
    "wh": [Phonemes.W, Phonemes.HH],
    "wi": [Phonemes.W, Phonemes.IY],
    "wj": [Phonemes.W, Phonemes.JH],
    "wk": [Phonemes.W, Phonemes.K],
    "wl": [Phonemes.W, Phonemes.L],
    "wm": [Phonemes.W, Phonemes.M],
    "wn": [Phonemes.W, Phonemes.N],
    "wo": [Phonemes.W, Phonemes.UH],
    "wp": [Phonemes.W, Phonemes.P],
    "wq": [Phonemes.W, Phonemes.K],
    "wr": [Phonemes.W, Phonemes.R],
    "ws": [Phonemes.W, Phonemes.S],
    "wt": [Phonemes.W, Phonemes.T],
    "wu": [Phonemes.W, Phonemes.UH],
    "wv": [Phonemes.W, Phonemes.V],
    "ww": [Phonemes.W],
    "wx": [Phonemes.W, Phonemes.K, Phonemes.S],
    "wy": [Phonemes.W, Phonemes.AY],
    "wz": [Phonemes.W, Phonemes.Z],
    "xa": [Phonemes.K, Phonemes.S],
    "xb": [Phonemes.K, Phonemes.S],
    "xc": [Phonemes.K, Phonemes.S],
    "xd": [Phonemes.K, Phonemes.S],
    "xe": [Phonemes.K, Phonemes.S],
    "xf": [Phonemes.K, Phonemes.S],
    "xg": [Phonemes.K, Phonemes.S],
    "xh": [Phonemes.K, Phonemes.S],
    "xi": [Phonemes.K, Phonemes.S],
    "xj": [Phonemes.K, Phonemes.S],
    "xk": [Phonemes.K, Phonemes.S],
    "xl": [Phonemes.K, Phonemes.S],
    "xm": [Phonemes.K, Phonemes.S],
    "xn": [Phonemes.K, Phonemes.S],
    "xo": [Phonemes.K, Phonemes.S],
    "xp": [Phonemes.K, Phonemes.S],
    "xq": [Phonemes.K, Phonemes.S],
    "xr": [Phonemes.K, Phonemes.S],
    "xs": [Phonemes.K, Phonemes.S],
    "xt": [Phonemes.K, Phonemes.S],
    "xu": [Phonemes.K, Phonemes.S],
    "xv": [Phonemes.K, Phonemes.S],
    "xw": [Phonemes.K, Phonemes.S],
    "xx": [Phonemes.K, Phonemes.S],
    "xy": [Phonemes.K, Phonemes.S],
    "xz": [Phonemes.K, Phonemes.S],
    "ya": [Phonemes.Y, Phonemes.AA],
    "yb": [Phonemes.Y, Phonemes.B],
    "yc": [Phonemes.Y, Phonemes.K],
    "yd": [Phonemes.Y, Phonemes.D],
    "ye": [Phonemes.Y, Phonemes.IY],
    "yf": [Phonemes.Y, Phonemes.F],
    "yg": [Phonemes.Y, Phonemes.G],
    "yh": [Phonemes.Y, Phonemes.HH],
    "yi": [Phonemes.Y, Phonemes.IY],
    "yj": [Phonemes.Y, Phonemes.JH],
    "yk": [Phonemes.Y, Phonemes.K],
    "yl": [Phonemes.Y, Phonemes.L],
    "ym": [Phonemes.Y, Phonemes.M],
    "yn": [Phonemes.Y, Phonemes.N],
    "yo": [Phonemes.Y, Phonemes.OW],
    "yp": [Phonemes.Y, Phonemes.P],
    "yq": [Phonemes.Y, Phonemes.K],
    "yr": [Phonemes.Y, Phonemes.R],
    "ys": [Phonemes.Y, Phonemes.S],
    "yt": [Phonemes.Y, Phonemes.T],
    "yu": [Phonemes.Y, Phonemes.UW],
    "yv": [Phonemes.Y, Phonemes.V],
    "yw": [Phonemes.Y, Phonemes.UW],
    "yx": [Phonemes.Y, Phonemes.K, Phonemes.S],
    "yy": [Phonemes.Y],
    "yz": [Phonemes.Y, Phonemes.Z],
    "za": [Phonemes.Z, Phonemes.AA],
    "zb": [Phonemes.Z, Phonemes.B],
    "zc": [Phonemes.Z, Phonemes.K],
    "zd": [Phonemes.Z, Phonemes.D],
    "ze": [Phonemes.Z, Phonemes.IY],
    "zf": [Phonemes.Z, Phonemes.F],
    "zg": [Phonemes.Z, Phonemes.G],
    "zh": [Phonemes.Z, Phonemes.HH],
    "zi": [Phonemes.Z, Phonemes.IY],
    "zj": [Phonemes.Z, Phonemes.JH],
    "zk": [Phonemes.Z, Phonemes.K],
    "zl": [Phonemes.Z, Phonemes.L],
    "zm": [Phonemes.Z, Phonemes.M],
    "zn": [Phonemes.Z, Phonemes.N],
    "zo": [Phonemes.Z, Phonemes.OW],
    "zp": [Phonemes.Z, Phonemes.P],
    "zq": [Phonemes.Z, Phonemes.K],
    "zr": [Phonemes.Z, Phonemes.R],
    "zs": [Phonemes.Z, Phonemes.S],
    "zt": [Phonemes.Z, Phonemes.T],
    "zu": [Phonemes.Z, Phonemes.UW],
    "zv": [Phonemes.Z, Phonemes.V],
    "zw": [Phonemes.Z, Phonemes.UW],
    "zx": [Phonemes.Z, Phonemes.K, Phonemes.S],
    "zy": [Phonemes.Z, Phonemes.AY],
    "zz": [Phonemes.Z]
}
